package jalview.appletgui;

import jalview.jbappletgui.*;
import jalview.schemes.*;
import jalview.datamodel.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class SliderPanel extends GSliderPanel
{
  AlignmentPanel ap;
  boolean forConservation = true;
  ColourSchemeI cs;

  static Frame conservationSlider;
  static Frame PIDSlider;

  public static int setConservationSlider(AlignmentPanel ap, ColourSchemeI cs, String source)
  {
    SliderPanel sp = null;
    ConservationColourScheme ccs = (ConservationColourScheme)cs;
    int value = ccs.inc;

    if(conservationSlider == null)
    {
      sp = new SliderPanel(ap, value, true, cs);
      conservationSlider = new Frame();
      conservationSlider.add(sp);
    }
    else
    {
        sp = (SliderPanel)conservationSlider.getComponent(0);
        sp.cs = cs;
        sp.setValue(value);
    }

    conservationSlider.setTitle("Conservation Colour Increment  ("+source+")");
    if(ap.av.alignment.getGroups()!=null)
      sp.setAllGroupsCheckEnabled( true );
    else
      sp.setAllGroupsCheckEnabled( false);

    return sp.getValue();
  }

  public static void showConservationSlider()
  {
    try{
      PIDSlider.setVisible(false);
      PIDSlider = null;
    }catch(Exception ex){}


    if(!conservationSlider.isVisible())
    {
      jalview.bin.JalviewLite.addFrame(conservationSlider, conservationSlider.getTitle(), 420, 100);
      conservationSlider.addWindowListener(new WindowAdapter()
      {
        public void windowClosing(WindowEvent e)
        {
          conservationSlider = null;
        }
      });

    }

  }

  public static int setPIDSliderSource(AlignmentPanel ap, ColourSchemeI cs, String source)
   {
     SliderPanel pid = null;
     if(PIDSlider == null)
     {
       pid = new SliderPanel(ap, 50, false, cs);
       PIDSlider = new Frame();
       PIDSlider.add(pid);
     }
     else
     {
         pid = (SliderPanel)PIDSlider.getComponent(0);
         pid.cs = cs;
     }
     PIDSlider.setTitle("Percentage Identity Threshold ("+source+")");

     if (ap.av.alignment.getGroups() != null)
       pid.setAllGroupsCheckEnabled(true);
     else
       pid.setAllGroupsCheckEnabled(false);

     return pid.getValue();

   }

   public static void showPIDSlider()
   {
     try{
       conservationSlider.setVisible(false);
       conservationSlider = null;
     }catch(Exception ex){}

     if(!PIDSlider.isVisible())
    {
      jalview.bin.JalviewLite.addFrame(PIDSlider, PIDSlider.getTitle(), 420, 100);
      PIDSlider.addWindowListener(new WindowAdapter()
      {
        public void windowClosing(WindowEvent e)
        {
          PIDSlider = null;
        }
      });
    }

   }


  public SliderPanel(AlignmentPanel ap, int value, boolean forConserve, ColourSchemeI cs)
  {
    this.ap = ap;
    this.cs = cs;
    forConservation = forConserve;
    undoButton.setVisible(false);
    applyButton.setVisible(false);
    if(forConservation)
    {
      label.setText("Modify conservation visibility");
      slider.setMinimum(0);
      slider.setMaximum(50+slider.getVisibleAmount());
      slider.setUnitIncrement(1);
    }
    else
    {
      label.setText("Colour residues above % occurence");
      slider.setMinimum(0);
      slider.setMaximum(100+slider.getVisibleAmount());
      slider.setBlockIncrement(1);
    }


    slider.addAdjustmentListener( new AdjustmentListener()
    {
      public void adjustmentValueChanged(AdjustmentEvent evt)
      {
        valueField.setText( slider.getValue()+"" );
        valueChanged( slider.getValue() );
      }
    });

    slider.setValue(value);
    valueField.setText(value+"");
  }

  public void valueChanged(int i)
  {
    if(cs == null)
      return;

    ColourSchemeI toChange = null;
    Vector allGroups = null;
    int groupIndex = 0;

    if(allGroupsCheck.getState())
    {
      allGroups = ap.av.alignment.getGroups();
      groupIndex = allGroups.size()-1;
    }
    else
      toChange = cs;


    while( groupIndex > -1 )
    {
      if(allGroups!=null)
        toChange = ((SequenceGroup)allGroups.elementAt(groupIndex)).cs;

      if (forConservation)
      {
        if (toChange instanceof ConservationColourScheme)
          ( (ConservationColourScheme) toChange).inc = i;
      }
      else
      {
        if (toChange.canThreshold())
          ( (ResidueColourScheme) toChange).setThreshold(i);
      }

      groupIndex--;
    }

    ap.seqPanel.seqCanvas.repaint();

  }

  public void setAllGroupsCheckEnabled(boolean b)
  {
    allGroupsCheck.setEnabled(b);
  }


  public void valueField_actionPerformed(ActionEvent e)
  {
    try{
      int i = Integer.parseInt(valueField.getText());
      slider.setValue(i);
    }
    catch(Exception ex)
    {
      valueField.setText( slider.getValue()+"" );
    }
  }

  public void setValue(int value)
  {
    slider.setValue(value);
  }
  public int getValue()
  {
    return Integer.parseInt(valueField.getText());
  }


}
